;     CP1251
; vim: set syntax=avr foldmethod=marker commentstring=;%s :

#define __SFR_OFFSET 0
#include <avr/io.h>

; Globals;{{{
.global play_mono8
.global play_mono16
.global play_stereo8
.global play_stereo16
.global dummy_isr
.global __vector_7
.global timer_isr
;}}}

; Macros ;{{{
.macro isr_exit
	lds		R25, saveR25
	lds		R24, saveR24
	lds		R31, saveR31
	lds		R30, saveSREG
	out		0x3F, R30
	lds		R30, saveR30
	reti
.endm

;}}}

.section .bss

;{{{
saveSREG:
	.byte 0
saveR24:
	.byte 0
saveR25:
	.byte 0
saveR22:
	.byte 0
saveR23:
	.byte 0
saveR30:
	.byte 0
saveR31:
	.byte 0

timer_isr:
	.byte 0, 0

;}}}

.section .text

__vector_7:
		sts		saveR30, R30
		in		R30, 0x3F
		sts		saveSREG, R30
		sts		saveR31, R31
		sts		saveR24, R24
		sts		saveR25, R25
		lds		R30, timer_isr
		lds		R31, timer_isr+1
		ijmp



play_mono8:
	; OCR1A += wav_period
	in		R24, OCR1AL
	in		R25, OCR1AH
	lds		R30, wav_period
	lds		R31, wav_period+1
	add		R24, R30
	adc		R25, R31
	out		OCR1AH, R25
	out		OCR1AL, R24

	; PORTD = wav_buf.wav_buf[wav_bufpos]
	; wav_bufpos = (wav_bufpos+1) & 1023

	lds		R24, wav_bufpos
	lds		R25, wav_bufpos+1
	adiw	R24, 1

	ldi		R30, lo8(wav_buf-1)
	ldi		R31, hi8(wav_buf-1)
	add		R30, R24
	adc		R31, R25

	andi	R25, 0x03
	sts		wav_bufpos, R24
	sts		wav_bufpos+1, R25

	ld		R24, Z
	out		PORTD, R24

	isr_exit

play_stereo8:
	; OCR1A += wav_period
	in		R24, OCR1AL
	in		R25, OCR1AH
	lds		R30, wav_period
	lds		R31, wav_period+1
	add		R24, R30
	adc		R25, R31
	out		OCR1AH, R25
	out		OCR1AL, R24

	; PORTD = (wav_buf.wav_buf[wav_bufpos] + wav_buf.wav_buf[wav_bufpos+1])/2
	; wav_bufpos = (wav_bufpos+2) & 1023

	lds		R24, wav_bufpos
	lds		R25, wav_bufpos+1
	adiw	R24, 2

	ldi		R30, lo8(wav_buf-2)
	ldi		R31, hi8(wav_buf-2)
	add		R30, R24
	adc		R31, R25

	andi	R25, 0x03
	sts		wav_bufpos, R24
	sts		wav_bufpos+1, R25

	ld		R24, Z+
	ld		R25, Z+
	add		R24, R25
	ror		R24
	out		PORTD, R24

	isr_exit

play_mono16:
	; OCR1A += wav_period
	in		R24, OCR1AL
	in		R25, OCR1AH
	lds		R30, wav_period
	lds		R31, wav_period+1
	add		R24, R30
	adc		R25, R31
	out		OCR1AH, R25
	out		OCR1AL, R24

	; PORTD = wav_buf.wav_buf[wav_bufpos]
	; wav_bufpos = (wav_bufpos+1) & 1023

	lds		R24, wav_bufpos
	lds		R25, wav_bufpos+1
	adiw	R24, 2

	ldi		R30, lo8(wav_buf-2)
	ldi		R31, hi8(wav_buf-2)
	add		R30, R24
	adc		R31, R25

	andi	R25, 0x03
	sts		wav_bufpos, R24
	sts		wav_bufpos+1, R25

	ldd		R24, Z+1
	ldi		R25, 0x80
	add		R24, R25
	out		PORTD, R24

	isr_exit


play_stereo16:
	sts		saveR22, R22
	sts		saveR23, R23
	; OCR1A += wav_period
	in		R24, OCR1AL
	in		R25, OCR1AH
	lds		R30, wav_period
	lds		R31, wav_period+1
	add		R24, R30
	adc		R25, R31
	out		OCR1AH, R25
	out		OCR1AL, R24

	; PORTD = (wav_buf.wav_buf[wav_bufpos:wav_bufpos+1] + wav_buf.wav_buf[wav_bufpos+2:wav_bufpos+3]) / 512
	; wav_bufpos = (wav_bufpos+4) & 1023

	lds		R24, wav_bufpos
	lds		R25, wav_bufpos+1
	adiw	R24, 4

	ldi		R30, lo8(wav_buf-4)
	ldi		R31, hi8(wav_buf-4)
	add		R30, R24
	adc		R31, R25

	andi	R25, 0x03
	sts		wav_bufpos, R24
	sts		wav_bufpos+1, R25

	ldd		R24, Z+1
	ldd		R25, Z+3
	ldi		R22, 0x80
	add		R24, R22
	add		R25, R22
	add		R25, R24
	ror		R25
	out		PORTD, R25

	lds		R23, saveR23
	lds		R22, saveR22
	isr_exit


dummy_isr:
	isr_exit



