#include "wav.h"
#include "task.h"
#include "status.h"
#include  <avr/io.h>
#include  <avr/interrupt.h>
#include  "interrupt.h"
#include  "params.h"

static TASK task;
static uint8_t status;
static uint8_t buf_empty;
static uint8_t opened;
static uint8_t volume = 3;

static uint8_t task_stop(void)
{
	cli();
	TIMSK &= ~_BV(OCIE1A);
	TIFR |= _BV(OCF1A);
	sei();
	if (opened) {
		f_close(&file);
		opened = 0;
	}
	status = S_READY;
	DDRA &= ~_BV(4);
	return 0;
}

static void set_volume(void)
{
	switch (volume) {
		case 0:
			DDRC |= _BV(6);
			DDRC |= _BV(7);
			break;
		case 1:
			DDRC &= ~_BV(6);
			DDRC |= _BV(7);
			break;
		case 2:
			DDRC |= _BV(6);
			DDRC &= ~_BV(7);
			break;
		default:
			DDRC &= ~(_BV(6) | _BV(7));
			break;
	}
}

extern void err(uint8_t);

static uint8_t task_start(uint8_t cmd, uint8_t* params, uint8_t len)
{
	char* fname = (char*)params;
	FRESULT res = f_open(&file, fname, FA_READ);
	if (FR_OK != res) {
		status = res;
		return 1;
	}
	uint16_t bytes_read;
	res = f_read(&file, &wav_buf.header, 512, &bytes_read);
	if (res) {
		status = res;
		f_close(&file);
		opened = 0;
		return 1;
	}
	if (bytes_read < 44) {
		status = S_INVALID_FILE;
		f_close(&file);
		opened = 0;
		return 1;
	}
	if (!parse_wav_header(&wav_buf.header)) {
		status = S_INVALID_FILE;
		f_close(&file);
		opened = 0;
		return 1;
	}
	if (wav_buf.header.format.bits_per_sample == 8) {
		if (wav_buf.header.format.channels == 1) {
			timer_isr = play_mono8;
		} else {
			timer_isr = play_stereo8;
		}
	} else {
		if (wav_buf.header.format.channels == 1) {
			timer_isr = play_mono16;
		} else {
			timer_isr = play_stereo16;
		}
	}
	buf_empty = 1;
	wav_bufpos = 44;
	status = S_PLAYING;
	opened = 1;
	DDRA |= _BV(4);
	set_volume();
	return 0;
}
	

static uint8_t task_run(void)
{
	uint16_t bytes_read = 1;
	FRESULT res = FR_OK;
	uint16_t pos;
	TIMSK |= _BV(OCIE1A);
	cli();
	pos = wav_bufpos;
	sei();
	if (buf_empty && pos < 512) {
		buf_empty = 0;
		res = f_read(&file, wav_buf.buf + 512, 512, &bytes_read);
	} else if (!buf_empty && pos >= 512) {
		buf_empty = 1;
		res = f_read(&file, wav_buf.buf, 512, &bytes_read);
	}
	if (res) {
		cli();
		TIMSK &= ~_BV(OCIE1A);
		TIFR |= _BV(OCF1A);
		sei();
		DDRA &= ~_BV(4);
		status = res;
		return 1;
	}
	if (!bytes_read) {
		cli();
		TIMSK &= ~_BV(OCIE1A);
		TIFR |= _BV(OCF1A);
		sei();
		DDRA &= ~_BV(4);
		status = S_READY;
		return 1;
	}
	return 0;
}

static uint8_t task_status(void)
{
	return status;
}

static uint8_t task_read(uint8_t param)
{
	return volume;
}

static uint8_t task_write(uint8_t param, uint8_t value)
{
	volume = value;
	set_volume();
	return 0;
}

TASK* init_player(void)
{
	task.start= task_start;
	task.stop = task_stop;
	task.run = task_run;
	task.status = task_status;
	task.read = task_read;
	task.write = task_write;
	return &task;
}

