/*     CP1251 */

/* vim: set foldmethod=marker : */

#include "tff.h"
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "diskio.h"
#include "status.h"
#include "wav.h"
#include "player.h"
#include "interrupt.h"
#include "i2c.h"
#include <string.h>
#include "params.h"


#define SLAVE_ADDRESS 0xAA
#define SLAVE_BUFSIZE 81

#define STARTUP_WAV "/startup.wav"

static volatile TASK* current_task;
static TASK* player_task;
static TASK* recorder_task;
static TASK* location_task;
volatile uint8_t buf_empty = 0;
static uint8_t i2c_buffer[SLAVE_BUFSIZE];
static uint8_t i2c_command;
static volatile uint8_t status = S_BUSY;
static uint8_t i2c_buffer2[SLAVE_BUFSIZE];
static volatile uint8_t buffer2_size = 0xFF;

FRESULT res;
FATFS work;

/* 
 *     interrupt ,
 *        sei
 *
 */

SIGNAL(SIG_OVERFLOW2) __attribute__ (( interrupt ));
SIGNAL(SIG_OVERFLOW2)
{
	disk_timerproc();
}


void err(uint8_t n)
{
	cli();
	uint8_t c = 0;
	DDRA |= _BV(4);
	while (n) {
		uint16_t i;
		for (i = 0; i < 50000; i++) {
			PORTD = c++;
		}
		for (i = 0; i < 50000; i++) {
			PORTD = PORTD;
		}
		n--;
	}
	for(;;);
}

uint32_t get_fattime(void)/*{{{*/
{
	return ((2008L-1980) << 25) | (1L << 21) | (1L << 16);
}/*}}}*/

static TASK* map_command(uint8_t i2c_command)/*{{{*/
{
	TASK* task = 0;
	i2c_command &= 0x30;
	if (i2c_command == PARAM_GROUP_PLAYER) {
		task = player_task;
	} else if (i2c_command == PARAM_GROUP_RECORDER) {
		task = recorder_task;
	} else if (i2c_command == PARAM_GROUP_LOCATION) {
		task = location_task;
	}
	return task;
}/*}}}*/

static void i2c_request_handler(void)/*{{{*/
{
	if (i2c_command == P_STATUS) {
		if (status == S_BUSY) {
			i2c_send(S_BUSY);
		}
		if (current_task) {
			i2c_send(current_task->status());
		} else {
			i2c_send(S_READY);
		}
		return;
	}
	TASK* task = map_command(i2c_command);
	if (task) {
		i2c_send(task->read(i2c_command));
	}
}/*}}}*/

static void i2c_rx_handler(void)/*{{{*/
{
	uint8_t avail = i2c_available();
	if (avail < 1) return;
	i2c_command = i2c_receive();
	if (avail == 1) {
		return;
	}
	if (i2c_command & 0x80) {
		if (status == S_BUSY) {
			return;
		}
		status = S_BUSY;
		uint8_t i;
		for (i = 0; i < avail-1; i++) {
			i2c_buffer2[i] = i2c_receive();
		}
		i2c_buffer2[avail-1] = 0;
		buffer2_size = avail;
		//err(avail);
	} else {
		TASK* task = map_command(i2c_command);
		if (task) {
			task->write(i2c_command, i2c_receive());
		}
	}
}/*}}}*/

int main(void)/*{{{*/
{
	/* - 1     */
	TCCR1B = 1 << CS10;


	/* - 2       256 */
	TCCR2 = (1 << CS22) | (1<<CS21);
	/*   - 2  8,9    
	 *     SD card
	 * */
	TIMSK |= _BV(TOIE2);

	/*  D    */
	DDRD = 0xFF;

	/*   */
	player_task = init_player();

	timer_isr = &dummy_isr;

	/*  I2C */
	i2c_setSlaveBuf(i2c_buffer, sizeof(i2c_buffer));
	i2c_onRX(i2c_rx_handler);
	i2c_onRequest(i2c_request_handler);
	i2c_init_slave(SLAVE_ADDRESS);

	sei();

	res = disk_initialize(0);
	res = f_mount(0, &work);
	res = f_open(&file, STARTUP_WAV, FA_READ);
	if (res == FR_OK) {
		f_close(&file);
		player_task->start(P_PLAY, (uint8_t*)STARTUP_WAV, 0);
		cli();
		current_task = player_task;
		sei();
	}
	status = S_READY;
	for(;;) {
		if (status == S_BUSY) {
			if (current_task) {
				current_task->stop();
			}
			TASK* task = map_command(i2c_command);
			if (task) {
				task->start(i2c_command, i2c_buffer2, buffer2_size);
				cli();
				current_task = task;
				sei();
			}
			status = S_READY;
		}

		if (current_task) {
			current_task->run();
		}
	}
}/*}}}*/

